/**************************************************************/
/*                                                            */
/*  CGI program to echo back the environment variables and    */
/*   input variables that have been submitted.                */
/*                                                            */
/*  This program is useful for debugging HTML forms and       */
/*   CGI scripts.                                             */
/*                                                            */
/*  Written by Jim MacNair - August, 1996                     */
/*                                                            */
/**************************************************************/

/**************************************************************/
/* Required include for C functions                           */
/**************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <locale.h>
#include "comsubs.h"

	char    InputBuffer[MAX_INPUT_CHAR];

/**************************************************************/
/* Routine to parse variables and convert them back to        */
/*  proper character sequences.                               */
/**************************************************************/

void PrintVar(char *Inpt)

{
	char *Value;

	/* Separate the variable name from the data  */
	Value = strchr(Inpt,'=');

	/* Convert = to string delimiter  */
	*Value++ = '\0';

	/* Convert to normal character sequences  */
	urlDecode(Inpt);
	urlDecode(Value);
	strcvrt(Value,'\n',' ');
	strcvrt(Value,'+',' ');

	/* Send the results back to the user */
	printf("%s = %s <p>\n",Inpt,Value);
}


/**************************************************************/
/* Routine to build the HTML header for the environment       */
/*  variables.                                                */
/**************************************************************/

void build_title()
{
    printf("<!doctype html public \"-//IETF//DTD HTML 2.0//EN\">\n");
    printf("<html><head>\n");
    printf("<title>Echo Environment and Input Variables</title>");
    printf("<p><p>\n\n");
    printf("</head>\n");
    printf("<body>\n");
    printf("<hr><p>\n");
    printf("<h2>Environment Variables<h2>\n");
    printf("<hr><p>\n");
}

/**************************************************************/
/* Routine to display the contents of a particular            */
/*  environment variable.                                     */
/**************************************************************/

void Show_Var(char *VarName)

{
	char    *p;

	p = getenv(VarName);
	printf("%s = %s<p>\n",VarName,p);
}

/**************************************************************/
/* Routine to display the contents of the environment         */
/*  variables.                                                */
/**************************************************************/

void Show_Environment()

{
	/* Display the common environment variables */
	Show_Var("ACCEPT");
	Show_Var("ACCEPT_ENCODING");
	Show_Var("ACCEPT_LANGUAGE");
	Show_Var("AUTH_TYPE");
	Show_Var("AUTHORIZATION");
	Show_Var("CHARGE_TO");
	Show_Var("CONTENT_FILE");
	Show_Var("CONTENT_LENGTH");
	Show_Var("CONTENT_TYPE");
	Show_Var("FROM");
	Show_Var("GATEWAY_INTERFACE");
	Show_Var("IF_MODIFIED_SINCE");
	Show_Var("HTTP_ACCEPT");
	Show_Var("HTTP_USER_AGENT");
	Show_Var("OUTPUT_FILE");
	Show_Var("PATH_INFO");
	Show_Var("PATH_TRANSLATED");
	Show_Var("PRAGMA");
	Show_Var("QUERY_STRING");
	Show_Var("REFERER");
	Show_Var("REMOTE_ADDR");
	Show_Var("REMOTE_IDENT");
	Show_Var("REMOTE_USER");
	Show_Var("REQUEST_LINE");
	Show_Var("REQUEST_METHOD");
	Show_Var("SCRIPT_NAME");
	Show_Var("SERVER_NAME");
	Show_Var("SERVER_PORT");
	Show_Var("SERVER_PROTOCOL");
	Show_Var("SERVER_SOFTWARE");
	Show_Var("USER_AGENT");

	/* Print the Form Variables Header */
	printf("<hr><p>\n");
	printf("<h2>Form Variables<h2><p>\n");
	printf("<hr><p>\n");
}

/**************************************************************/
/* Main part of the program.                                  */
/**************************************************************/

int main(void)

{
        int ContentLength;
        int i;
        int x;
        char *RequestMethod;
        char *p;

      /* Turn buffering off for stdout  */
        setvbuf(stdout, NULL, _IONBF, 0);

      /* Send the document header  */
        build_header();
		build_title();

      /* Display the environment variable contents. */
        Show_Environment();

      /* Find out how we were invoked  */
        RequestMethod = getenv("REQUEST_METHOD");

      /* Get the data from the client  */
        if (strcmp(RequestMethod,"POST") == 0)
         {
           /* Find out how much input data there is. */
           p = getenv("CONTENT_LENGTH");
           if (p != NULL)
                ContentLength = atoi(p);
              else
                ContentLength = 0;

           if (ContentLength > sizeof(InputBuffer) -1)
                ContentLength = sizeof(InputBuffer) -1;

           /* Now, read in the data */
           i = 0;
           while (i < ContentLength)
            {
                x = fgetc(stdin);
                if (x == EOF)
                        break;
                InputBuffer[i++] = x;
            }

           /* Add a valid string terminator */
           InputBuffer[i] = '\0';
           ContentLength = i;

           /* Get the input format type */
           p = getenv("CONTENT_TYPE");
           if (p == NULL)
                return(1);      /* Something is wrong! */

           if (strcmp(p, "application/x-www-form-urlencoded") == 0)
            {
              /* Parse the input data into variables */
              p = strtok(InputBuffer, "&");
              while (p != NULL)
               {
                 PrintVar(p);
                 p = strtok(NULL, "&");
               }
            }
         }

      /* Get the data from the client  */
        if (strcmp(RequestMethod,"GET") == 0)
         {
          p = getenv("QUERY_STRING");
          if (p == NULL)
            printf("No Form variables found");
           else
            {
              p = strtok(p, "&");
              while (p != NULL)
               {
                 PrintVar(p);
                 p = strtok(NULL, "&");
               }
            }
         }

        build_trailer();
        return(0);
}
