// vartable.cpp: implementation of the Cvartable class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "vartable.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

////////////////////////////////////////////////////////////////
//                                                            
// Variable name and value array definition                   
//                                                            
// Input variables are saved in this array.  Other variables  
//  can be added as well.  To build the output HTML page,     
//  the file containing the HTML page is written to           
//  stdout, with variable names being substituted with the    
//  corresponding values.  Variables are recognized by        
//  looking for a variable name delimited by a beginning
//  ampersand and a trailing semi-colon.                          
//                                                            
////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Cvartable::Cvartable()

{
	pVars = (VARSTRUCT *)malloc(sizeof(VARSTRUCT) * (MAX_VAR_COUNT + 1));
    memset(pVars,'\0',(sizeof(VARSTRUCT) * (MAX_VAR_COUNT + 1)));
}

Cvartable::~Cvartable()

{
	free(pVars);
}

////////////////////////////////////////////////////////////////
//                                                            
// Routine to look up a variable in the variable table
// and return a pointer to the variable data.  This pointer
// should only be considered valid until the next time an
// entry is added or updated.  If the variable name is not
// found, a NULL pointer will be returned.
//
////////////////////////////////////////////////////////////////

const char * Cvartable::GetVarPtr(const char *varname)

{
	int i = 0;
	char *ptr = NULL;

	// Make sure we don't have a null pointer
	ISAPIASSERT(varname != NULL);

	// Search for the variable name 
	while ((i < MAX_VAR_COUNT) && (strlen(pVars[i].VarName) != 0) &&
			strcmp(pVars[i].VarName,varname))
	{
		i++;
	} // endwhile 

	
	// check if we found it 
	if (strlen(pVars[i].VarData) > 0)
	{
		ptr = pVars[i].VarData;
	}

	// return either a pointer to the variable or NULL if not found 
	return ptr;
}

////////////////////////////////////////////////////////////////
//                                                            
// Routine to add a variable name and data value to the
// variable table.  If the variable name is already in
// the table, the current data will be updated. 
//
////////////////////////////////////////////////////////////////

void Cvartable::AddVar(const char *varname, const char *vardata)

{
	int i = 0;

	// Make sure we don't have a null pointer
	ISAPIASSERT(varname != NULL);
	ISAPIASSERT(vardata != NULL);
	ISAPIASSERT(strlen(varname) < MAX_NAME_LEN);
	ISAPIASSERT(strlen(vardata) < MAX_DATA_LEN);

	while ((i < MAX_VAR_COUNT) && (strlen(pVars[i].VarName) > 0) &&
			(strcmp(pVars[i].VarName, varname) != 0))
	{
		i++;
	} // endwhile 

	strcpy(pVars[i].VarName,varname);
	strcpy(pVars[i].VarData,vardata);
}

////////////////////////////////////////////////////////////////
//                                                            
// Routine to update the value of an existing variable name.
// If the variable name is not currently in the table, the
// name will be inserted into the table.
//
////////////////////////////////////////////////////////////////

void Cvartable::UpdateVar(const char *varname, const char *vardata)

{
	// Use the AddVar routine, since this does the same thing 
	AddVar(varname, vardata);
}

////////////////////////////////////////////////////////////////
//                                                            
// This routine will return the current data value of a
// variable. 
//
////////////////////////////////////////////////////////////////

void Cvartable::GetVarVal(const char * varname, char * vardata)

{
	// Make sure we don't have a null pointer
	ISAPIASSERT(varname != NULL);
	ISAPIASSERT(vardata != NULL);

	const char *ptr = GetVarPtr(varname);

	// Indicate that the variable was not found 
	vardata[0] = '\0';

	// check if we found the variable 
	if (ptr != NULL)
	{
		// copy the data to the variable 
		strcpy(vardata,ptr);
	}
}

////////////////////////////////////////////////////////////////
//                                                            
// Routine to build the HTML page                             
//                                                            
//  This routine will read a file and write it out, using
//  the ISAPI context handle.  It will     
//  substitute variable values for any variable names       
//  found which are enclosed between '&' and ';' characters.  
//  The file name should point to a string with the name of 
//  a valid HTML template file that the ISAPI extension will
//  use to generate the response HTML page.
//                                                            
////////////////////////////////////////////////////////////////

int Cvartable::write_html_page(CHttpServerContext * pCtxt, char * filename)

{
    FILE *htmlfile;
    char *pct1;
    char *pct2;
    char HTML_Line[MAX_LINE_LEN];
    char temp_line[MAX_LINE_LEN];


    if ((htmlfile = fopen(filename, "r")) == NULL)
     {
		// Exit the routine indicating unable to open the template file
		return 100;
     }

    while (fgets(HTML_Line, MAX_LINE_LEN, htmlfile) != NULL)
     {
      pct1 = HTML_Line;
      while (pct1 != 0)
       {
        pct1 = strchr(pct1, '&');
        if (pct1 != 0)
         {
          pct2 = strchr(pct1 + 1, ';');

		  // Check for no trailing delimiter 
		  if (pct2 == 0)
		  {
			  // If no trailing delimiter, then prevent an infinite loop 
			  pct1 = 0;
		  }

          if ((pct2 != 0) && ((pct2 - pct1) < (MAX_NAME_LEN + 1)))
          {
            // Save the end of the line 
            strcpy(temp_line, pct2 + 1);

            // Make the variable name a proper string 
            *pct2 = '\0';

            // Get the variable data into the HTML line 
            GetVarVal(pct1 + 1, pct1);

            // Check for characters which must be escaped 
		    /*ExpandChar(pct1, '&', "&amp;");
			ExpandChar(pct1, '<', "&lt;");
            ExpandChar(pct1, '>', "&gt;");*/
	        //ExpandChar(pct1, '%', "&#37");
		    //ExpandChar(pct1, '\"', "&quot");

			// Replace the end of the line 
			strcat(pct1, temp_line);
          } // endif 
         } // endif 
       } // endwhile 

		// Ignore NULL lines 
		if (strlen(HTML_Line) > 0)
		{
			// Write out the HTML line from the template file 
			*pCtxt << HTML_Line;
		}
     } // endwhile 

    fclose(htmlfile);
	return 0;
}

////////////////////////////////////////////////////////////////
//                                                            
// Routine to replace a character with another character      
//  or characters.  This is used primarily to insert escape   
//  sequences into output data streams.                       
//                                                            
////////////////////////////////////////////////////////////////

void Cvartable::ExpandChar(char * VarData, char ch, char * NewValue)

{
	char temp_line[MAX_LINE_LEN];

	VarData = strchr(VarData, ch);

	while (VarData != 0)
    {
		// Save the end of the line 
		strcpy(temp_line, VarData + 1);

		// Copy in the substitute value 
		strcpy(VarData, NewValue);

		// Move pointer past character substitution 
		VarData = VarData + strlen(NewValue);

		// Replace the end of the line
		strcat(VarData, temp_line);

		// Search for more characters 
		VarData = strchr(VarData, ch);
	} // endwhile 
}
